PluginWrapper.registerPlugin('example_object_list', {
	name: 'Object List Example Plugin',
	element: {
		minSize: {width: 100, height: 100},
		defaultSize: {width: 200, height: 100},
		resizable: true,
	},
	
	propertyDialog: {
		tabs: [
			{name: 'Tab 1', children: [
				{type: 'HorizontalLayout', children: [
					{type: 'VerticalLayout', children: [
						{type: 'Label', text: 'Age'},
						{type: 'TextField', id: 'age', init: function(def, fields) { this._elem.draggable(); }}
					]},
					{type: 'VerticalLayout', children: [
						{type: 'Label', text: 'Name'},
						{type: 'TextField', id: 'name'}
					]},
					{type: 'Button', text: 'Add', id: 'btnAdd', css: {marginTop: 25}}
				]},
				{type: 'CustomContainer', id: 'list', css: {marginTop: 15}, init: function(def, fields) {
				    var thisSelf = this;
				    this._elem.sortable({
				        stop: function(e, ui) {
				            fields.reloadListObj();
				        }
				    });
				}}
			]}
		]
	},

	openAction: function(fields, data, elem) {
		var thisSelf = this;
		if (!fields.eventsSet) {
			fields.eventsSet = true;
			fields.listObj = [];
			fields.btnAdd.on('click', function() {
				var nameValue = fields.name.getValue(),
					ageValue = fields.age.getValue();
				if (!nameValue || !ageValue) return;
				
				fields.listObj.push({ name: nameValue, age: ageValue });
				fields.name.setValue('');
				fields.age.setValue('');
				thisSelf.reloadList(fields);
			});
			fields.reloadListObj = function() {
			    fields.listObj = [];
			    fields.list._elem.children().each(function() {
			        var name = $(this).attr('data-name');
			        var age = $(this).attr('data-age');
			        fields.listObj.push({ name: name, age: age });
			    });
			};
		}
		fields.listObj = [];
		for (var i=0; i < data.content.list.length; i++) {
			fields.listObj.push(data.content.list[i]);
		}
		this.reloadList(fields);
	},

	reloadList: function(fields) {
		var thisSelf = this;
		fields.list._elem.empty();
		for (var i in fields.listObj) {
		    var name = fields.listObj[i].name;
		    var age = fields.listObj[i].age;
			var block = $('<div>').attr('data-name', name).attr('data-age', age);
			block.append($('<span>').html('<strong>Name:</strong> ' + name + 
					', <strong>Age:</strong> ' + age));
			block.append($('<button class="btn btn-sm btn-link">')
					.text('Remove')
					.css('margin-left', '4px')
					.on('click', function() {
						$(this).parent().remove();
						fields.reloadListObj();
					}));
			fields.list._elem.append(block);
		}
	},

	applyAction: function(fields, data, elem) {
		var list = [];
		for (var idx in fields.listObj) {
			list.push(fields.listObj[idx]);
		}
		data.content.list = list;
		data.content.listNotEmpty = (data.content.list.length > 0);
	},

	loadAction: function(data) {
		if (!data.content.list) data.content.list = [
			{name: 'Joe', age: 38},
			{name: 'Michael', age: 29},
			{name: 'Helen', age: 31}
		];
		data.content.listNotEmpty = (data.content.list.length > 0);
	}
});